# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by mktables.PL from e.g. Unicode.300.
# Any changes made here will be lost!
return <<'END';
0030	0039
00b2	00b3
00b9	
06f0	06f9
2070	
2074	2079
2080	2089
2460	249b
24ea	
ff10	ff19
END
