ARCHNAME='"vos"'
BIN='"/system/ported/command_library"'
BIN_EXP='"/system/ported/command_library"'
BYTEORDER='0x4321'
CAN_PROTOTYPE='define'
CASTFLAGS='0'
CASTNEGFLOAT='define'
CAT2(a,b)='a ## b'
CPPMINUS='"-"'
CPPSTDIN='"cc -E"'
CPPRUN='"cc -E -"'
DB_Hash_t='int'
DB_Prefix_t='int'
DOUBLESIZE='8'
Direntry_t='struct dirent'
Drand01()='rand()/(RAND_MAX+1)'
FILE_base(fp)='((fp)->_base)'
FILE_bufsiz(fp)='((fp)->_cnt + (fp)->_ptr - (fp)->_base)'
FILE_cnt(fp)='((fp)->_cnt)'
FILE_ptr(fp)='((fp)->_ptr)'
FLEXFILENAMES='define'
Fpos_t='fpos_t'
Free_t='void'
Gconvert(x,n,t,b)='sprintf((b),"%.*g",(n),(x))'
Gid_t='gid_t'
Gid_t_f='"%d"'
Gid_t_size='4'
Groups_t='gid_t'
HASCONST='define'
HASVOLATILE='define'
HAS_ALARM='define'
HAS_DBL_DIG='define'
HAS_DIFFTIME='define'
HAS_FCHMOD='define'
HAS_FCNTL='define'
HAS_FGETPOS='define'
HAS_FPATHCONF='define'
HAS_FSETPOS='define'
HAS_GETHOSTBYADDR='define'
HAS_GETHOSTBYNAME='define'
HAS_GETHOSTENT='define'
HAS_GETHOST_PROTOS='define'
HAS_GETLOGIN='define'
HAS_GETNETBYADDR='define'
HAS_GETNETBYNAME='define'
HAS_GETNETENT='define'
HAS_GETNET_PROTOS='define'
HAS_GETPPID='define'
HAS_GETPROTOBYNAME='define'
HAS_GETPROTOBYNUMBER='define'
HAS_GETPROTOENT='define'
HAS_GETPROTO_PROTOS='define'
HAS_GETSERVBYNAME='define'
HAS_GETSERVBYPORT='define'
HAS_GETSERVENT='define'
HAS_GETSERV_PROTOS='define'
HAS_HTONL='define'
HAS_HTONS='define'
HAS_ISASCII='define'
HAS_LDBL_DIG='define'
HAS_LOCALECONV='define'
HAS_LOCKF='define'
HAS_LONG_DOUBLE='define'
HAS_LSTAT='define'
HAS_MBLEN='define'
HAS_MBSTOWCS='define'
HAS_MBTOWC='define'
HAS_MEMCMP='define'
HAS_MEMCPY='define'
HAS_MEMMOVE='define'
HAS_MEMSET='define'
HAS_MKDIR='define'
HAS_MKFIFO='define'
HAS_MKTIME='define'
HAS_NTOHL='define'
HAS_NTOHS='define'
HAS_OPEN3='define'
HAS_PATHCONF='define'
HAS_PIPE='define'
HAS_POLL='define'
HAS_READDIR='define'
HAS_READLINK='define'
HAS_RENAME='define'
HAS_REWINDDIR='define'
HAS_RMDIR='define'
HAS_SANE_MEMCMP='/ **/'
HAS_SELECT='define'
HAS_SETHOSTENT='define'
HAS_SETLOCALE='define'
HAS_SETNETENT='define'
HAS_SETPROTOENT='define'
HAS_SETSERVENT='define'
HAS_SETVBUF='define'
HAS_SHMAT_PROTOTYPE='define'
HAS_SOCKET='define'
HAS_STRCHR='define'
HAS_STRCOLL='define'
HAS_STRERROR='define'
HAS_STRTOD='define'
HAS_STRTOL='define'
HAS_STRTOUL='define'
HAS_STRXFRM='define'
HAS_SYMLINK='define'
HAS_SYSCONF='define'
HAS_SYSTEM='define'
HAS_SYS_ERRLIST='define'
HAS_TIMES='define'
HAS_UMASK='define'
HAS_UNAME='define'
HAS_VPRINTF='define'
HAS_WAITPID='define'
HAS_WCSTOMBS='define'
HAS_WCTOMB='define'
I16SIZE='2'
I16TYPE='short'
I32SIZE='4'
I32TYPE='int'
I8SIZE='1'
I8TYPE='char'
INTSIZE='4'
IVSIZE='4'
IVTYPE='int'
IVdf='"%d"'
I_DIRENT='define'
I_FCNTL='define'
I_FLOAT='define'
I_LIMITS='define'
I_LOCALE='define'
I_MATH='define'
I_NETDB='define'
I_STDARG='define'
I_STDDEF='define'
I_STDLIB='define'
I_STRING='define'
I_SYS_IOCTL='define'
I_SYS_STAT='define'
I_SYS_TIME='define'
I_SYS_TIMES='define'
I_SYS_TYPES='define'
I_SYS_WAIT='define'
I_UNISTD='define'
I_UTIME='define'
I_VALUES='define'
LOC_SED='"/system/ported/command_library/sed.pm"'
LONGSIZE='4'
LONG_DOUBLESIZE='8'
LSEEKSIZE='4'
MEM_ALIGNBYTES='8'
M_VOID='define'
Malloc_t='void *'
Mode_t='mode_t'
NVSIZE='8'
NVTYPE='double'
NV_PRESERVES_UV='define'
Netdb_hlen_t='int'
Netdb_host_t='char *'
Netdb_name_t='char *'
Netdb_net_t='long'
OSNAME='"VOS"'
Off_t='off_t'
PERL_PRIfldbl='"%f"'
PERL_PRIgldbl='"%g"'
PRIVLIB='"/system/ported/perl"'
PRIVLIB_EXP='"/system/ported/perl"'
PTRSIZE='4'
Pid_t='pid_t'
RANDBITS='15'
Rand_seed_t='unsigned int *'
RD_NODATA='-1'
seedfunc='srand'
SELECT_MIN_BITS='32'
SETUID_SCRIPTS_ARE_SECURE_NOW='define'
SHORTSIZE='2'
SH_PATH='"/bin/sh"'
SIG_NAME='"ZERO","ABRT","FPE","ILL","INT","SEGV","TERM","USR1","USR2","IO","POLL","HUP","URG","ALRM","KILL","PIPE","QUIT","CHLD","CONT","STOP","TSTP","TTIN","TTOU","BUS","RT1","RT2","RT3","RT4","RT5","RT6","RT7","RT8",0'
SIG_NUM='0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,0'
SITEARCH='"/system/ported/site_command_library"'
SITEARCH_EXP='"/system/ported/site_command_library"'
SITELIB='"/system/ported/site_command_library"'
SITELIB_EXP='"/system/ported/site_command_library"'
SSize_t='ssize_t'
STARTPERL='"!perl.pm"'
STDCHAR='unsigned char'
STDIO_CNT_LVALUE='define'
STDIO_PTR_LVALUE='define'
STRINGIFY(a)='StGiFy(a)'
Select_fd_set_t='fd_set *'
Shmat_t='void *'
Sigjmp_buf='jmp_buf'
Siglongjmp(buf,retval)='longjmp((buf),(retval))'
Signal_t='void'
Sigsetjmp(buf,save_mask)='setjmp((buf))'
Size_t='size_t'
StGiFy(a)='# a'
Strerror(e)='strerror(e)'
Time_t='time_t'
Timeval='struct timeval'
U16SIZE='2'
U16TYPE='unsigned short'
U32SIZE='4'
U32TYPE='unsigned int'
U8SIZE='1'
U8TYPE='unsigned char'
USE_STDIO_BASE='define'
USE_STDIO_PTR='define'
USE_STRUCT_COPY='define'
UVSIZE='4'
UVTYPE='unsigned int'
UVof='"%o"'
UVuf='"%u"'
UVxf='"%x"'
Uid_t='uid_t'
Uid_t_f='4'
Uid_t_size='"%d"'
VAL_EAGAIN='EAGAIN'
VAL_O_NONBLOCK='O_NONBLOCK'
VOIDFLAGS='15'
VOIDUSED='15'
_(args)='()'
void='int'
