package less;

=head1 NAME

less - perl pragma to request less of something from the compiler

=head1 SYNOPSIS

    use less;  # unimplemented

=head1 DESCRIPTION

Currently unimplemented, this may someday be a compiler directive
to make certain trade-offs, such as perhaps

    use less 'memory';
    use less 'CPU';
    use less 'fat';


=cut

1;
